# -*- coding: utf-8 -*-

import os
import sys
import tempfile
import nwdiag.command
from nwdiag.elements import *


def extra_case(func):
    filename = "VL-PGothic-Regular.ttf"
    testdir = os.path.dirname(__file__)
    pathname = "%s/truetype/%s" % (testdir, filename)

    if os.path.exists(pathname):
        func.__test__ = True
    else:
        func.__test__ = False

    return func


def __build_diagram(filename, format, *args):
    testdir = os.path.dirname(__file__)
    diagpath = "%s/diagrams/%s" % (testdir, filename)

    fontfile = "VL-PGothic-Regular.ttf"
    fontpath = "%s/truetype/%s" % (testdir, fontfile)

    try:
        argv = sys.argv
        tmpfile = tempfile.mkstemp()[1]
        sys.argv = ['nwdiag.py', '-T', format, '-f', fontpath,
                    '-o', tmpfile, diagpath]
        if args:
            sys.argv += args

        DiagramNode.clear()
        DiagramEdge.clear()
        NodeGroup.clear()

        nwdiag.command.main()
    finally:
        sys.argv = argv
        os.unlink(tmpfile)


def diagram_files():
    testdir = os.path.dirname(__file__)
    pathname = "%s/diagrams/" % testdir

    skipped = ['errors',
               'white.gif']

    return [d for d in os.listdir(pathname) if d not in skipped]


@extra_case
def test_generator():
    formats = ['svg', 'png']
    try:
        import reportlab.pdfgen.canvas
        formats.append('pdf')
    except ImportError:
        sys.stderr.write("Skip testing about pdf exporting.\n")
        pass

    for diagram in diagram_files():
        for format in formats:
            yield __build_diagram, diagram, format

            if format == 'png':
                yield __build_diagram, diagram, format, '--antialias'



# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import math
import blockdiag.DiagramMetrix
from blockdiag import elements
from blockdiag.utils.namedtuple import namedtuple
from blockdiag.utils.XY import XY


class DiagramMetrix(blockdiag.DiagramMetrix.DiagramMetrix):
    def __init__(self, diagram, **kwargs):
        super(DiagramMetrix, self).__init__(diagram, **kwargs)

        self.setdefault('networks', diagram.networks)

        if diagram.page_padding is None and kwargs.get('pagePadding') is None:
            top_padding = self['spanHeight']
            bottom_padding = self['spanHeight'] / 3
            left_padding = (self['nodeWidth'] + self['spanWidth']) * 3 / 4

            margin = self['cellSize'] * 6
            self['pagePadding'] = [top_padding + margin, margin,
                                   bottom_padding + margin,
                                   left_padding + margin]

        self['pageMargin'] = XY(0, 0)

    def originalMetrix(self):
        kwargs = {}
        for key in self:
            kwargs[key] = self[key]
        kwargs['scale_ratio'] = 1

        return DiagramMetrix(self, **kwargs)

    def node(self, node):
        metrix = super(DiagramMetrix, self).node(node)
        return NodeMetrix(node, metrix)

    def cell(self, node):
        if isinstance(node, elements.NodeGroup):
            metrix = GroupMetrix(node, self)
        else:
            metrix = super(DiagramMetrix, self).cell(node)
        return metrix

    def network(self, network):
        return NetworkMetrix(network, self)


class NetworkMetrix(blockdiag.DiagramMetrix.NodeMetrix):
    @property
    def trunkline(self):
        x0 = self.left().x
        x1 = self.right().x
        y = self.top().y

        return [XY(x0, y), XY(x1, y)]

    def top(self):
        pt = super(NetworkMetrix, self).top()
        return XY(pt.x, pt.y - self.metrix.spanHeight / 2)

    def left(self):
        pt = super(NetworkMetrix, self).left()
        return XY(pt.x - self.metrix.spanWidth / 2, pt.y)

    def right(self):
        pt = super(NetworkMetrix, self).right()
        return XY(pt.x + self.metrix.spanWidth / 2, pt.y)

    @property
    def textbox(self):
        x = self.left().x
        y = self.top().y

        width = self.metrix.nodeWidth * 3 / 2
        height = self.metrix.nodeHeight

        return (x - width, y - height / 2, x, y + height / 2)


class NodeMetrix(object):
    def __init__(self, node, metrix):
        self.node = node
        self.metrix = metrix

    def top(self):
        return self.metrix.top()

    def bottom(self):
        return self.metrix.bottom()

    @property
    def connectors(self):
        m = self.metrix.metrix

        above = [n for n in self.node.networks if n.xy.y <= self.node.xy.y]
        above.sort(lambda a, b: -cmp(a.xy.y, b.xy.y))

        bottom = [n for n in self.node.networks if n.xy.y > self.node.xy.y]
        bottom.sort(lambda a, b: cmp(a.xy.y, b.xy.y))

        Connector = namedtuple('Connector', 'network line textbox jumps')

        for networks in [above, bottom]:
            for network in networks:
                if network.xy.y <= self.node.xy.y:
                    x, y2 = self.top()
                    y1 = m.network(network).top().y
                else:
                    x, y1 = self.bottom()
                    y2 = m.network(network).top().y

                if len(networks) == 1:
                    dx = 0
                else:
                    pos = networks.index(network)
                    base_x = (len(networks) - 1) / 2.0 - pos
                    dx = int(math.floor(base_x * m.cellSize * 2))

                width = m.nodeWidth + m.spanWidth
                textbox = [x + dx - m.cellSize, y2 - m.spanHeight / 2,
                           x + width, y2]
                line = [XY(x + dx, y1), XY(x + dx, y2)]

                yield Connector(network, line, textbox, [])


class GroupMetrix(blockdiag.DiagramMetrix.NodeMetrix):
    def __init__(self, node, metrix):
        super(GroupMetrix, self).__init__(node, metrix)

        self.is_root_group = False
        if node.nodes:
            network = min(node.nodes[0].networks, key=lambda n: n.xy.y)
            if self.top().x == metrix.network(network).top().x:
                self.is_root_group = True

    def groupLabelBox(self):
        box = super(GroupMetrix, self).groupLabelBox()
        span = self.metrix.spanHeight / 2 - self.metrix.cellSize
        box = (box[0], box[1] - span, box[2], box[3] - span)

        if self.is_root_group:
            width = box[2] - box[1]
            box = (box[0] + width / 2, box[1], box[2], box[3])

        return box

    def marginBox(self):
        box = super(GroupMetrix, self).marginBox()
        margin_x = self.metrix.spanHeight / 2
        margin_y = self.metrix.cellSize
        return (box[0] - margin_y, box[1] - margin_x,
                box[2] + margin_y, box[3] + margin_x)



# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import re
import sys
import blockdiag.elements
from blockdiag.elements import *
from blockdiag.utils.XY import XY


class Diagram(blockdiag.elements.Diagram):
    def __init__(self):
        super(Diagram, self).__init__()

        self.orientation = 'portrait'
        self.node_width = 104
        self.span_width = 48
        self.span_height = 104
        self.groups = []
        self.networks = []

    def fixiate(self):
        if len(self.nodes) + len(self.networks) > 0:
            nodes = self.nodes + self.networks
            self.width = max(x.xy.x + x.width for x in nodes)
            self.height = max(x.xy.y + x.height for x in nodes)


class DiagramNode(blockdiag.elements.DiagramNode):
    def __init__(self, id):
        super(DiagramNode, self).__init__(id)

        self.address = {}
        self.networks = []

    def set_attributes(self, network, attrs):
        for attr in attrs:
            if attr.name == 'address':
                address = re.sub('\s*,\s*', '\n', unquote(attr.value))
                self.address[network] = address
            else:
                self.set_attribute(attr)


class Network(blockdiag.elements.NodeGroup):
    def __init__(self, id):
        super(Network, self).__init__(id)

        self.address = None
        self.width = 0
        self.height = 0

    @property
    def display_label(self):
        if self.label:
            if self.address:
                label = "%s\n%s" % (self.label, self.address)
            else:
                label = self.label
        else:
            label = self.address

        return label


class NodeGroup(blockdiag.elements.NodeGroup):
    def __init__(self, id):
        super(NodeGroup, self).__init__(id)

        self.layouted = False



# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import re
import sys
from ConfigParser import SafeConfigParser
from optparse import OptionParser
import nwdiag
import DiagramDraw
import diagparser
from blockdiag import utils
from blockdiag.command import detectfont
from builder import ScreenNodeBuilder


def parse_option():
    version = "%%prog %s" % nwdiag.__version__
    usage = "usage: %prog [options] infile"
    p = OptionParser(usage=usage, version=version)
    p.add_option('-a', '--antialias', action='store_true',
                 help='Pass diagram image to anti-alias filter')
    p.add_option('-c', '--config',
                 help='read configurations from FILE', metavar='FILE')
    p.add_option('-o', dest='filename',
                 help='write diagram to FILE', metavar='FILE')
    p.add_option('-f', '--font', default=[], action='append',
                 help='use FONT to draw diagram', metavar='FONT')
    p.add_option('-P', '--pdb', dest='pdb', action='store_true', default=False,
                 help='Drop into debugger on exception')
    p.add_option('-T', dest='type', default='PNG',
                 help='Output diagram as TYPE format')
    options, args = p.parse_args()

    if len(args) == 0:
        p.print_help()
        sys.exit(0)

    options.type = options.type.upper()
    if not options.type in ('SVG', 'PNG', 'PDF'):
        msg = "ERROR: unknown format: %s\n" % options.type
        sys.stderr.write(msg)
        sys.exit(0)

    if options.type == 'PDF':
        try:
            import reportlab.pdfgen.canvas
        except ImportError:
            msg = "ERROR: colud not output PDF format; Install reportlab\n"
            sys.stderr.write(msg)
            sys.exit(0)

    if options.config and not os.path.isfile(options.config):
        msg = "ERROR: config file is not found: %s\n" % options.config
        sys.stderr.write(msg)
        sys.exit(0)

    configpath = options.config or "%s/.nwdiagrc" % os.environ.get('HOME')
    if os.path.isfile(configpath):
        config = SafeConfigParser()
        config.read(configpath)

        if config.has_option('nwdiag', 'fontpath'):
            fontpath = config.get('nwdiag', 'fontpath')
            options.font.append(fontpath)

    return options, args


def main():
    options, args = parse_option()

    infile = args[0]
    if options.filename:
        outfile = options.filename
    elif infile == '-':
        outfile = 'output.' + options.type.lower()
    else:
        outfile = re.sub('\..*', '', infile) + '.' + options.type.lower()

    if options.pdb:
        sys.excepthook = utils.postmortem

    fontpath = detectfont(options)

    try:
        if infile == '-':
            import codecs
            stream = codecs.getreader('utf-8')(sys.stdin)
            tree = diagparser.parse_string(stream.read())
        else:
            tree = diagparser.parse_file(infile)

        diagram = ScreenNodeBuilder.build(tree)

        draw = DiagramDraw.DiagramDraw(options.type, diagram, outfile,
                                       font=fontpath,
                                       antialias=options.antialias)
        draw.draw()
        draw.save()
    except UnicodeEncodeError, e:
        msg = "ERROR: UnicodeEncodeError caught (check your font settings)\n"
        sys.stderr.write(msg)
    except Exception, e:
        sys.stderr.write("ERROR: %s\n" % e)


if __name__ == '__main__':
    main()
